pacman::p_load(tidyverse, viper, mixtools, ggpubr)

# Regulon --------------------------------------------------------------------

# Notice that the "pvalue" column, index and header should be removed from the network file genereated by the ARACNe-AP to meet the input requirement of viper.
# This problem exits in the version we use, but may be fixed in the future.
# Refrence: https://support.bioconductor.org/p/123998/

# Process the network file
net <- read_tsv("Temp/ARACNeAP/network.txt") %>%
    subset(select = -pvalue) 
write.table(net, file = "Temp/net.txt", sep = "\t", row.names = F, col.names = F, quote = F)

# Convert the network to the regulon object
regulon <- aracne2regulon(afile = "Temp/net.txt", eset = paste0("../Data/RNA_GSE12452.tsv"), format = "3col")
save(regulon, file = "Temp/regulon.R")


# Expression matrix and grouping information ------------------------------------

# Read the expression matrix
rna <- read_tsv("../Data/RNA_GSE12452.tsv") %>%
    column_to_rownames(var = "Gene Symbol") %>%
    as.data.frame()

# Grouping information
rna_ref <- subset(rna, select = c(1:10)) %>%
    as.matrix()
rna_test <- subset(rna, select = -c(1:10)) %>%
    as.matrix()
cli <- rbind(
    data.frame(Sample = colnames(rna_ref), category = rep("normal", times = 10)),
    data.frame(Sample = colnames(rna_test), category = rep("npc", times = 31))
)

# msVIPER: Master Regulators --------------------------------------------------------------------

# Generate the NULL model by sample permutations
nullmodel <- ttestNull(x = rna_test, y = rna_ref, per = 1000, repos = TRUE, verbose = FALSE)
save(nullmodel, file = paste0("Temp/msVIPER-Nullmodel.R"))

# Calculate the signature
signature <- rowTtest(x = rna_test, y = rna_ref)
signature <- (qnorm(signature$p.value / 2, lower.tail = FALSE) * sign(signature$statistic))[, 1]

# Find the Master Regulators (MRs)
mrs <- msviper(signature, regulon, nullmodel,
    minsize = 25, verbose = F, cores = 1, level = 10, synergy = 0,
    adaptive.size = T,
    ges.filter = TRUE,
    pleiotropy = T, pleiotropyArgs = list(
        regulators = 0.05,
        shadow = 0.05,
        targets = 10,
        penalty = 20,
        method = "adaptive"
    )
)

# Results visualization 
png(paste0("Output/msVIPER-mrs_plot.png"), width = 4000, height = 6000, res=300)
plot(mrs, cex = .7, pval = T)
dev.off()

# Leading edge analysis
mrs <- ledge(mrs)
summary(mrs, mrs = 50)

save(mrs, file = paste0("Output/msVIPER-mrs.R"))
write.csv(summary(mrs, mrs = 1500), paste0("Output/msVIPER-mrs.csv"), quote = F)


# VIPER: NES Matrix --------------------------------------------------------------------
vpres <- viper(rna, regulon, verbose = FALSE)
dim(vpres)
save(vpres, file = paste0("Temp/VIPER-nes_matrix.R"))

